package com.mt.wms.gateway.filter;

import com.alibaba.fastjson.JSON;
import com.mt.wms.core.dto.LoginUser;
import com.mt.wms.core.vo.R;
import com.mt.wms.gateway.config.GatewayConfig;
import com.mt.wms.gateway.feign.PassportService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;

import javax.servlet.*;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author jiff
 * @date 2019/11/27
 * @since 1.0
 */
@Slf4j
@WebFilter(urlPatterns = "/*", filterName = "gatewayFilter")
public class GatewayFilter implements Filter {
    @Autowired
    @Lazy()
    private PassportService passportService;

    @Autowired
    @Lazy
    private GatewayConfig gatewayConfig;

    private static final String GATEWAY_PREFIX = "/api";

    /**
     * Called by the web container to indicate to a filter that it is being
     * placed into service. The servlet container calls the init method exactly
     * once after instantiating the filter. The init method must complete
     * successfully before the filter is asked to do any filtering work.
     * <p>
     * The web container cannot place the filter into service if the init method
     * either:
     * <ul>
     * <li>Throws a ServletException</li>
     * <li>Does not return within a time period defined by the web
     *     container</li>
     * </ul>
     *
     * @param filterConfig The configuration information associated with the
     *                     filter instance being initialised
     * @throws ServletException if the initialisation fails
     */
    @Override
    public void init(FilterConfig filterConfig) throws ServletException {

    }

    /**
     * The <code>doFilter</code> method of the Filter is called by the container
     * each time a request/response pair is passed through the chain due to a
     * client request for a resource at the end of the chain. The FilterChain
     * passed in to this method allows the Filter to pass on the request and
     * response to the next entity in the chain.
     * <p>
     * A typical implementation of this method would follow the following
     * pattern:- <br>
     * 1. Examine the request<br>
     * 2. Optionally wrap the request object with a custom implementation to
     * filter content or headers for input filtering <br>
     * 3. Optionally wrap the response object with a custom implementation to
     * filter content or headers for output filtering <br>
     * 4. a) <strong>Either</strong> invoke the next entity in the chain using
     * the FilterChain object (<code>chain.doFilter()</code>), <br>
     * 4. b) <strong>or</strong> not pass on the request/response pair to the
     * next entity in the filter chain to block the request processing<br>
     * 5. Directly set headers on the response after invocation of the next
     * entity in the filter chain.
     *
     * @param request  The request to process
     * @param response The response associated with the request
     * @param chain    Provides access to the next filter in the chain for this
     *                 filter to pass the request and response to for further
     *                 processing
     * @throws IOException      if an I/O error occurs during this filter's
     *                          processing of the request
     * @throws ServletException if the processing fails for any other reason
     */
    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
        HttpServletRequest httpRequest = ((HttpServletRequest) request);
        String requestUri = httpRequest.getRequestURI();
        if (requestUri.startsWith(GATEWAY_PREFIX)) {
            requestUri = requestUri.substring(GATEWAY_PREFIX.length());
        }
        final String method = httpRequest.getMethod();
        //忽略鉴权的地址
        if (isIgnoreAuth(requestUri)) {
            if (log.isDebugEnabled()) {
                log.debug("忽略鉴权的地址={}", requestUri);
            }
            chain.doFilter(request, response);
            return;
        }
        R<LoginUser> r = passportService.authorization(requestUri, method);

        if (r != null && r.ok()) {
            if (r.getData() != null) {
                //不打印
                //log.info("loginUser={}", JSON.toJSONString(r.getData()));
            }
            chain.doFilter(request, response);
        } else {
            r = r != null ? r : R.failed();
            response.setContentType("application/json;charset=UTF-8");
            response.setCharacterEncoding("UTF-8");
            response.getWriter().print(JSON.toJSONString(r));
        }
    }

    /**
     * Called by the web container to indicate to a filter that it is being
     * taken out of service. This method is only called once all threads within
     * the filter's doFilter method have exited or after a timeout period has
     * passed. After the web container calls this method, it will not call the
     * doFilter method again on this instance of the filter. <br>
     * <br>
     * <p>
     * This method gives the filter an opportunity to clean up any resources
     * that are being held (for example, memory, file handles, threads) and make
     * sure that any persistent state is synchronized with the filter's current
     * state in memory.
     */
    @Override
    public void destroy() {

    }

    private boolean isIgnoreAuth(String requestUri) {
        for (String ignore : gatewayConfig.getIgnoreAuths()) {
            if (log.isDebugEnabled()) {
                log.debug("ignore auth url is {}", ignore);
            }
            // 编译正则表达式，忽略大小写的写法
            Pattern pattern = Pattern.compile(ignore, Pattern.CASE_INSENSITIVE);
            Matcher matcher = pattern.matcher(requestUri);
            // 查找字符串中是否有匹配正则表达式的字符/字符串
            if (matcher.find()) {
                return true;
            }
        }
        return false;
    }
}
